/**********************************************************************
* ?2007 Microchip Technology Inc.
*
* FileName:        OC_Simple_PWM.c
* Dependencies:    Header (.h) files if applicable, see below
* Processor:       PIC24Fxxxx
* Compiler:        MPLAB?C30 v3.00 or higher
*
* SOFTWARE LICENSE AGREEMENT:
* Microchip Technology Incorporated ("Microchip") retains all 
* ownership and intellectual property rights in the code accompanying
* this message and in all derivatives hereto.  You may use this code,
* and any derivatives created by any person or entity by or on your 
* behalf, exclusively with Microchip's proprietary products.  Your 
* acceptance and/or use of this code constitutes agreement to the 
* terms and conditions of this notice.
*
* CODE ACCOMPANYING THIS MESSAGE IS SUPPLIED BY MICROCHIP "AS IS". NO 
* WARRANTIES, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT 
* NOT LIMITED TO, IMPLIED WARRANTIES OF NON-INFRINGEMENT, 
* MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE APPLY TO THIS 
* CODE, ITS INTERACTION WITH MICROCHIP'S PRODUCTS, COMBINATION WITH 
* ANY OTHER PRODUCTS, OR USE IN ANY APPLICATION. 
*
* YOU ACKNOWLEDGE AND AGREE THAT, IN NO EVENT, SHALL MICROCHIP BE 
* LIABLE, WHETHER IN CONTRACT, WARRANTY, TORT (INCLUDING NEGLIGENCE OR
* BREACH OF STATUTORY DUTY), STRICT LIABILITY, INDEMNITY, 
* CONTRIBUTION, OR OTHERWISE, FOR ANY INDIRECT, SPECIAL, PUNITIVE, 
* EXEMPLARY, INCIDENTAL OR CONSEQUENTIAL LOSS, DAMAGE, FOR COST OR 
* EXPENSE OF ANY KIND WHATSOEVER RELATED TO THE CODE, HOWSOEVER 
* CAUSED, EVEN IF MICROCHIP HAS BEEN ADVISED OF THE POSSIBILITY OR THE
* DAMAGES ARE FORESEEABLE.  TO THE FULLEST EXTENT ALLOWABLE BY LAW, 
* MICROCHIP'S TOTAL LIABILITY ON ALL CLAIMS IN ANY WAY RELATED TO THIS
* CODE, SHALL NOT EXCEED THE PRICE YOU PAID DIRECTLY TO MICROCHIP 
* SPECIFICALLY TO HAVE THIS CODE DEVELOPED.
*
* You agree that you are solely responsible for testing the code and 
* determining its suitability.  Microchip has no obligation to modify,
* test, certify, or support the code.
*
* REVISION HISTORY:
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
* Author        Date      	Comments on this revision
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
* Albert Z.		12/26/08	Original Release
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*
* ADDITIONAL NOTES:
* Pin 81 on Explorer 16 with PIC24FJ128GA010 PIM or PIC24FJ64GA004 PIM 
* is a simple PWM output. 
*
* This code example is tested on PIC24FJ128GA010 and PIC24FJ64GA004. 
* 
* PIC24FJ256GA110 family and PIC24FJ256GB110 family have a different 
* Output Compare module, please read datasheet for more detail. 
* The Family Reference Manual has two sections about Output Compare. 
* For PIC24FJ256GA110 and PIC24FJ256GB110 family, please refer 
* Section 35. Output Compare with Dedicated Timer. 
* For PIC24FJ128GA010 and PIC24FJ64GA004 family, please refer 
* Section 16. Output Compare. 
**********************************************************************/

#include "p24fxxxx.h"

#if defined (__PIC24FJ64GA004__)	//Defined by MPLAB when using 24FJ64GA004 device
	_CONFIG1( JTAGEN_OFF & GCP_OFF & GWRP_OFF & COE_OFF & FWDTEN_OFF & ICS_PGx1) 
	_CONFIG2( FCKSM_CSDCMD & OSCIOFNC_OFF & POSCMOD_HS & FNOSC_PRIPLL & I2C1SEL_SEC & IOL1WAY_ON)
#elif defined (__PIC24FJ128GA010__)	//Default to the PIC24FJ128GA010
	// JTAG/Code Protect/Write Protect/Clip-on Emulation mode
	// Watchdog Timer/ICD pins select
	_CONFIG1(JTAGEN_OFF & GCP_OFF & GWRP_OFF & COE_OFF & FWDTEN_OFF & ICS_PGx2) 
	// Disable CLK switch and CLK monitor, OSCO or Fosc/2, HS oscillator,
	// Primary oscillator
	_CONFIG2(FCKSM_CSDCMD & OSCIOFNC_OFF & POSCMOD_HS & FNOSC_PRIPLL)
#endif

#define OC5_IO	22			// define OC5 output function number for PPS

unsigned char pwmFHtimes;	// PWM frequency hopping times, it is based on PWM cycle

int main (void)
{
	// Disable Watch Dog Timer
	RCONbits.SWDTEN = 0;

	pwmFHtimes = 0;
	
	// PPS
	#ifdef __PIC24FJ64GA004__
		// Unlock Registers
		__builtin_write_OSCCONL(OSCCON & 0xbf);
		// Configure Output Functions *********************
		//RP14 = OC5
		RPOR7bits.RP14R = OC5_IO;
		// Lock Registers
		__builtin_write_OSCCONL(OSCCON | 0x40);
	#endif

	/*  
	The following code example will set the Output Compare 1 module
	for PWM mode w/o FAULT pin enabled, a 50% duty cycle and 
	a PWM frequency of 52.08 kHz at Fosc = 32 MHz. Timer 2 is selected 
	as the clock for the PWM time base and Timer2 interrupts are enabled.
	 
	Find the Period register value for a desired PWM frequency of 52.08 kHz, 
	where Fosc = 8 MHz with PLL (32 MHz device clock rate) and 
	a Timer2 prescaler setting of 1:1. 
	Tcy = 2/Fosc = 62.5 ns
	PWM Period   =  1/PWM Frequency = 1/52.08 kHz = 19.2 s
	PWM Period   = (PR2 + 1) * Tcy * (Timer 2 Prescale Value)
	19.2 s = (PR2 + 1) * 62.5 ns * 1 
	PR2 = 306
	*/
	OC5CONbits.OCM		= 0; 		// Output compare channel is disabled
	OC5R				= 0x0099;	// Initialize Compare Register1 with 50% duty cycle
	OC5RS				= 0x0099;	// Initialize Secondary Compare Register1 with 50% duty cycle
	OC5CONbits.OCSIDL	= 0;		// Output capture will continue to operate in CPU Idle mode
	OC5CONbits.OCFLT	= 0;		// No PWM Fault condition has occurred (this bit is only used when OCM<2:0> = 111)
	OC5CONbits.OCTSEL	= 0;		// Timer2 is the clock source for output Compare
	OC5CONbits.OCM		= 0x6;		// PWM mode on OC, Fault pin disabled
	PR2					= 0x0132;	// Initialize PR2 with 0x0132 = 0d306 as PWM cycle
	IFS0bits.T2IF		= 0;		// Clear Output Compare interrupt flag
	IEC0bits.T2IE		= 1;		// Enable Output Compare interrupts
	T2CONbits.TON		= 1;		// Start Timer2 with assumed settings

	while(1)
	{
		Idle();
	}

	return 0;
}

// Example code for Timer2 ISR: PWM frequency hopping every 5 cycles. 
// Please use oscilloscope to probe on pin 81 of PIC24FJ PIM on 
// Explorer 16 Development Board. You can see the duty cycle will be 
// updated between 50% duty cycle and 75% duty cycle every 5 cycles. 
void __attribute__ ((__interrupt__, no_auto_psv)) _T2Interrupt(void)
{
	IFS0bits.T2IF = 0;
	
	if (++pwmFHtimes == 5) {	// every 5 cycles for update
		pwmFHtimes = 0;
		// update the duty cycle for PWM
		if (OC5RS == 0x0099)
			OC5RS = 0x00E5;	// 75% duty cycle
		else
			OC5RS = 0x0099;	// 50% duty cycle
	}
}
